angular.module('webchat', [])

    /**
     * WebService Client factory
     */
    .factory('WSClient', function () {
        /**
         * WebSocket connection
         *
         * @type {WebSocket}
         */
        var connection = null;

        /**
         * CSTA toolkit
         *
         * @type {CstaToolkit}
         */
        var toolkit = null;

        /**
         * Session Id
         *
         * @type {string}
         */
        var sessionId = '';
		
        //if ('sessionStorage' in window) {
            // restore session if already was established one
            //var lastSessionDate = window.sessionStorage.getItem('lastSessionDate');
            //var date = new Date();
            //if (lastSessionDate && (date.getTime()-lastSessionDate)/1000 < 180) {
            //    sessionId = window.sessionStorage.getItem('sessionId') || '';
            //} else {
            //    window.sessionStorage.clear();
        //    }
        //}

        var config = {
            wsUri: null, // URI of webservice
            clientName: null, // Client's name
            pageName: null, // Page name
            group: "Group1" // Group name
        };

        /**
         * List of event handlers
         *
         * @type {object}
         */
        var eventHandlers = {
            'open': [],
            'close': [],
            'error': [],
            'message': [],
            'status': [],
            'config': [],
            'partner': [],
            'other': []
        };

        // need to send sessionId right after connection is open
        eventHandlers.open.push(function () {
            connection.send(JSON.stringify({
                session_id: sessionId
            }));
        });

        /**
         * WebSocket Client
         */
        var wsClient = (function (config) {
            var self = this;

            /**
             * Init CSTA toolkit
             */
            var initToolktit = function () {
                if (!toolkit) {
                    toolkit = new CstaToolkit();
                }
            };

            /**
             * Call handlers from list
             *
             * @param eventName Name of event
             * @param eventData Object to pass to handlers
             */
            var callEventHandlers = function (eventName, eventData) {
                //console.log('Event '+eventName, eventData);
                if (eventHandlers[eventName]) {
                    for (var i=0; i<eventHandlers[eventName].length; i++) {
                        eventHandlers[eventName][i](eventData);
                    }
                }
            };

            /**
             * Check if message is a session token
             *
             * @param message
             * @returns {boolean}
             */
            var isSessionToken = function (message) {
                return (message.indexOf("session_valid") != -1);
            };

            /**
             * WebSocket onMessage event handler
             *
             * @param event
             */
            var onMessage = function (event) {
                if (isSessionToken(event.data)) {
                    proceedSession(event.data);
                    //var data = angular.$eval(event.data);
                    //if (data.session_id && 'sessionStorage' in window) {
                    //    sessionId = data.session_id;
                    //    var date = new Date();
                    //    sessionStorage.setItem('sessionId', sessionId);
                    //    sessionStorage.setItem('lastSessionDate', date.getTime());
                    //}
                } else {
                    toolkit.parseData(event.data);
                }
            };

            /**
             * Dummy event, for future development
             *
             * @param event
             */
            var dummyEvent = function (event) {
                callEventHandlers('other', event);
            };

            /**
             * Setup session
             */
            var proceedSession = function () {
                toolkit.setupEventCallbacks(
                    dummyEvent.bind(wsClient),
                    dummyEvent.bind(wsClient),
                    dummyEvent.bind(wsClient),
                    dummyEvent.bind(wsClient),//parked,
                    dummyEvent.bind(wsClient),//held,
                    dummyEvent.bind(wsClient),//retrieved,
                    dummyEvent.bind(wsClient),//assigned,
                    dummyEvent.bind(wsClient),//presence, //TODO - move it to lineManager?
                    dummyEvent.bind(wsClient),//csta_error,
                    dummyEvent.bind(wsClient),//network_error
                    dummyEvent.bind(wsClient),
                    function (event) {
                        callEventHandlers('status', event);
                    },
                    function (event) {
                        callEventHandlers('config', event);
                    },
                    function (event) {
                        callEventHandlers('partner', event);
                    },
                    function (event) {
                        callEventHandlers('message', event);
                    }
                );

                var package = toolkit.createWebChatStart(
                    config.group,
                    config.clientName,
                    config.pageName,
                    "",
                    function () {}
                );
                connection.send(package);
            };

            return {
                /**
                 * Initialize client
                 */
                init: function () {
                    if ('WebSocket' in window) {
                        initToolktit();
                        connection = new WebSocket(config.wsUri);
                        connection.onopen = function () {
                            callEventHandlers('open', {});
                        };
                        connection.onclose = function () {
                            callEventHandlers('close', {});
                        };
                        connection.onerror = function () {
                            callEventHandlers('error', {});
                        };
                        connection.onmessage = onMessage.bind(wsClient);
                    } else {
                        callEventHandlers('error', {
                            message: 'WebSocket not supported'
                        });
                    }
                },

                /**
                 * Send message
                 *
                 * @param message
                 */
                sendMessage: function (message) {
                    var package = toolkit.createWebChatMsg(
                        message,
                        function (data) {
                            // response callback
                            //console.log('Message response', data);
                        }
                    );
                    //console.log(package);
                    connection.send(package);
                }
            };
        })(config);

        /**
         * This client API will be exposed
         *
         * @type {{init: Function, sendMessage: Function}}
         */
        var externalWSClient = {
            /**
             * Initialize client
             *
             * @param parameters
             * @param handlers
             */
            init: function (parameters, handlers) {
                config = $.extend(config, parameters)
                wsClient.init();
                for (var i in eventHandlers) {
                    if (handlers[i]) {
                        eventHandlers[i].push(handlers[i]);
                    }
                }
            },

            /**
             * Send message
             *
             * @param message
             */
            sendMessage: function (message) {
                wsClient.sendMessage(message);
            }
        };
        
        return externalWSClient;
    })
    
    .controller('WebChat', ['$scope', '$timeout', 'WSClient', '$location', function WebChat($scope, $timeout, WSClient, $location) {
        var parameters = $location.search();
        var loading = false;
        var operatorName = null;
		var agentCount = 0;
        $scope.statuses = {
            CONNECTING: 1,
            CONNECTED: 2,
            DISCONNECTED: 4,
            ERROR: 5
        };
        $scope.message = '';
        $scope.messages = [];
        $scope.connection = {
            status: $scope.statuses.CONNECTING,
            error: null
        };

        $scope.sendMessage = function () {
            loading = true;
            $scope.messages.push({
                from: parameters.clientName,
                message: $scope.message,
                date: new Date(),
                isOperator: false
            });
            WSClient.sendMessage($scope.message);
            $scope.message = '';
            loading = false;
        };

        $scope.allowSendMessage = function () {
            return $scope.connection.status === $scope.statuses.CONNECTED && !loading;
        };

        WSClient.init(parameters, {
            error: function () {
                $scope.connection.status = $scope.statuses.ERROR;
                $scope.$digest();
            },
            open: function () {
                $scope.connection.status = $scope.statuses.CONNECTING;
                $scope.$digest();
            },
            close: function () {
                if ($scope.connection.status !== $scope.statuses.ERROR) {
                    $scope.connection.status = $scope.statuses.DISCONNECTED;
                    $scope.$digest();
                }
            },
            message: function (event) {
                $scope.messages.push({
                    from: event.name,
                    message: event.msg,
                    date: new Date(),
                    isOperator: true
                });
                $scope.$digest();
            },
            status: function (event) {
                if (event.status === 'DISCONNECTED') {
                    $scope.connection.status = $scope.statuses.DISCONNECTED;
                }
                $scope.$digest();
            },
            partner: function (event) {
                if (event.statusNew === 'PARTNER_CONNECTED') {
					agentCount++;
                    $scope.connection.status = $scope.statuses.CONNECTED;
                    operatorName = event.name;
                    WSClient.sendMessage('Intro text: '+parameters.text+'; Page name: '+parameters.pageName);
                } else if (event.statusNew === 'PARTNER_DISCONNECTED') {
					agentCount--;
					WSClient.sendMessage('Agent left: ' + event.name);
                    operatorName = null;
					//no agent remains
					if (WSClient.agentCount == 0){
						$scope.connection.status = $scope.statuses.CONNECTING;
					}
                }
                $scope.$digest();
            }
        });
    }])

    .directive('autoScroll', ['$timeout', function($timeout) {

        function link(scope, element, attrs) {

            function updateScroll() {
                element.scrollTop(element[0].scrollHeight);
            }

            scope.$watch(attrs.autoScroll, function(value) {
                updateScroll();
                $timeout(updateScroll, 100);
            });
        }

        return {
            link: link
        };
    }])

    .directive('actionOnEnter', function() {
        function link(scope, element, attrs) {
            element.keydown(function (event) {
                if (event.keyCode == 13 && attrs.actionOnEnter) {
                    if (event.ctrlKey) {
                        var val = this.value;
                        if (typeof this.selectionStart == "number" && typeof this.selectionEnd == "number") {
                            var start = this.selectionStart;
                            this.value = val.slice(0, start) + "\n" + val.slice(this.selectionEnd);
                            this.selectionStart = this.selectionEnd = start + 1;
                        } else if (document.selection && document.selection.createRange) {
                            this.focus();
                            var range = document.selection.createRange();
                            range.text = "\r\n";
                            range.collapse(false);
                            range.select();
                        }
                    } else {
                        scope.$eval(attrs.actionOnEnter);
                        scope.$digest();
                    }
                    return false;
                }
            });
        }

        return {
            link: link
        };
    });